%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <values.h>
#include "stype.h"

#if defined(__MSDOS__) || defined(__WIN32__)
#define SLASH '\\'
#ifdef __TURBOC__
#ifndef __BORLANDC__
#define alloca	salloc
#else
#include <malloc.h>
#endif
#endif
#else
#define SLASH '/'
#endif

#define INVALID		MAXDOUBLE
#define MAX_LIST_LEN	128
#define MIN_DELTA_PARAM 0.001
#define MAX_EXPLODE_INFINITY 1e16
#define MIN_SIGMA	0.0
#define MAX_SIGMA	1.0
#define MAX_STRING_SIZE 1024
#define DEFAULT_NO_PARAMETER 10
#define FILENAME_POSTFIX_SIZE 3

#ifndef DEBUG 
#define DEBUG 0
#endif

extern void *salloc(size_t);
extern int compare(const void *, const void *);
extern FILE *yyin;
extern int line_nr;

int error=0;
long no_pb_times=0, no_snapshot_times=0, list_idx=0;
int needbetarange=0;
char strg[MAX_STRING_SIZE];

char *program[] = { "histogrm", "lyap_exp", "rot_numb", "pullback", 
		    "flow_bwd", "traject",  "traj_bwd", "bwd_expl", 
		    "lyaponed", "exploned", "flowoned" };
enum       { HISTOGRAM_PROC=0, LYAPUNOV_PROC, ROTNUMBER_PROC, PULLBACK_PROC, 
	    FLOW_BWD_PROC, TRAJECTORY_PROC, TRAJECTORY_BWD_PROC, 
	    BACKWARD_EXPLOSION_TIMES_PROC, LYAPUNOV_ONED_PROC, 
	     BWD_EXPLOSION_TIMES_ONED_PROC, FLOW_BWD_ONED_PROC} procedure;

FILE       *fp_log;
char       *filename;
int        filenamesize=0;
long       gauss_seed=66239;
double     t_end=10;
double     dt=0.001;
double     x_s=1.0,y_s=1.0;
double     lx_s=0.4,ly_s=0.4;
double     nlx1_s=1.0, nlx2_s=1.0;
double     nly1_s=1.0, nly2_s=-1.0;
long       no_bins_x=10;
long       no_bins_y=10;
long       no_values_x=5;
long       no_values_y=5;
double     x_low=0, x_high=5;
double     y_low=0, y_high=5;
double     alpha=1, beta=2, sigma=0.5;
double     beta_start=INVALID, beta_end=INVALID;
double     delta_param=INVALID;
double     time_to_find_grid_dimensions=100;
double     time_for_relaxation=100;
double     t_until_stationary=100;
double     explode_infinity=MAX_EXPLODE_INFINITY;
int        grid_fixed_flag=1;
double     value_list[MAX_LIST_LEN];
double     snapshot_times[MAX_LIST_LEN];
double     pb_times[MAX_LIST_LEN];

int compare(const void *v1, const void *v2) {
	const double *d1, *d2;

	d1 = v1;
	d2 = v2;

	if (*d1 > *d2) return  1;
	if (*d1 < *d2) return -1;

	return 0;
}
%}

%token IVAL FVAL STRING

%token PROCEDURE SEED  T_END  TIME_STEPSIZE INITIAL_VALUE_X INITIAL_VALUE_Y
%token X_RANGE Y_RANGE 
%token NUMBER_OF_BINS_X NUMBER_OF_BINS_Y
%token NUMBER_OF_VALUES_X NUMBER_OF_VALUES_Y
%token ALPHA BETA SIGMA PARAMETER_STEPSIZE
%token TIME_TO_FIND_GRID_DIMENSIONS
%token TIME_FOR_RELAXATION_OF_TRAJECTORY
%token GRID_FIXED
%token IVALUE_LINEARIZED_SYSTEM_X IVALUE_LINEARIZED_SYSTEM_Y
%token IVALUE_NONLINEAR_SYSTEM_X1 IVALUE_NONLINEAR_SYSTEM_X2
%token IVALUE_NONLINEAR_SYSTEM_Y1 IVALUE_NONLINEAR_SYSTEM_Y2
%token TIME_TO_REACH_STATIONARY_SOLUTION
%token PULLBACK_TIMES SNAPSHOT_TIMES
%token FILENAME EXPLODE_INFINITY

%token TRUE FALSE

%token HISTOGRAM LYAPUNOV TRAJECTORY TRAJECTORY_BWD ROTNUMBER
%token PULLBACK FLOW_BWD
%token BACKWARD_EXPLOSION_TIMES
%token LYAPUNOV_ONED BWD_EXPLOSION_TIMES_ONED FLOW_BWD_ONED


%%

Sde_Parameter : PROCEDURE '=' Procedure ';' Parameter_List_Opt
;

Procedure       : HISTOGRAM                  { procedure = HISTOGRAM_PROC; }
                | LYAPUNOV                   { procedure = LYAPUNOV_PROC;  
				                 needbetarange  = 1;
			                     }
                | ROTNUMBER                  { procedure = ROTNUMBER_PROC;  
				                 needbetarange  = 1;
			                     }
		| PULLBACK                   { procedure = PULLBACK_PROC; }
		| FLOW_BWD                   { procedure = FLOW_BWD_PROC; }
                | TRAJECTORY                 { procedure = TRAJECTORY_PROC; }
		| TRAJECTORY_BWD             { procedure = TRAJECTORY_BWD_PROC; }
                | BACKWARD_EXPLOSION_TIMES   { procedure = BACKWARD_EXPLOSION_TIMES_PROC; }
		| LYAPUNOV_ONED              { procedure = LYAPUNOV_ONED_PROC;  
				                 needbetarange  = 1;
			                     }
                | BWD_EXPLOSION_TIMES_ONED   { procedure = BWD_EXPLOSION_TIMES_ONED_PROC; }
		| FLOW_BWD_ONED              { procedure = FLOW_BWD_ONED_PROC; }
;

Parameter_List_Opt : 
	           | Parameter_List
;
 
Parameter_List: Parameter ';'
              | Parameter_List Parameter ';' 
;

Parameter: SEED '=' IVAL { static int done=0;
 
                           if (done) yyerror("Multiple declaration.");
			   gauss_seed = $3.ival;
			   done = 1;
                         }
         | T_END '=' Value
			{ static int done=0;

                          if (done) yyerror("Multiple declaration.");
			  t_end = fabs($3.fval);
			  done = 1;
	                }
	| TIME_STEPSIZE '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   dt = fabs($3.fval);
			   done = 1;
	                }
	| INITIAL_VALUE_X '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   x_s = $3.fval;
			   done = 1;
	                }
	| INITIAL_VALUE_Y '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   y_s = $3.fval;
			   done = 1;
	                }
        | IVALUE_LINEARIZED_SYSTEM_X '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   lx_s = $3.fval;
			   done = 1;
	                }
	| IVALUE_LINEARIZED_SYSTEM_Y '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   ly_s = $3.fval;
			   done = 1;
	                }
	| IVALUE_NONLINEAR_SYSTEM_X1 '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   nlx1_s = $3.fval;
			   done = 1;
	                }
	| IVALUE_NONLINEAR_SYSTEM_X2 '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   nlx2_s = $3.fval;
			   done = 1;
	                }
	| IVALUE_NONLINEAR_SYSTEM_Y1 '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   nly1_s = $3.fval;
			   done = 1;
	                }
	| IVALUE_NONLINEAR_SYSTEM_Y2 '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   nly2_s = $3.fval;
			   done = 1;
	                }
	| X_RANGE '=' '[' Value ',' Value ']'  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   if ($4.fval < $6.fval) {
				x_low = $4.fval;	
				x_high = $6.fval;
			   } else {
				x_low = $6.fval;
				x_high = $4.fval;
			   }
			   if (x_low == x_high) {
	      yyerror("Illegal range specification. Specify [Value_1,Value_2] with Value_1 not equal Value_2.");
			   }
			   done = 1;
	                }
	| Y_RANGE '=' '[' Value ',' Value ']'  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   if ($4.fval < $6.fval) {
				y_low = $4.fval;	
				y_high = $6.fval;
			   } else {
				y_low = $6.fval;
				y_high = $4.fval;
			   }
			   if (y_low == y_high) {
		      yyerror("Illegal range specification. Specify [Value_1,Value_2] with Value_1 not equal Value_2.");
			   }
			   done = 1;
	                }
	| NUMBER_OF_BINS_X '=' IVAL 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   no_bins_x = $3.ival;
			   if (no_bins_x < 2) {
			     yyerror("Illegal specification. NUMBER_OF_BINS_X must be larger than 1.");
			   }
			   done = 1;
                         }
	| NUMBER_OF_BINS_Y '=' IVAL 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   no_bins_y = $3.ival;
			   if (no_bins_y < 2) {
			     yyerror("Illegal specification. NUMBER_OF_BINS_Y must be larger than 1.");
			   }
			   done = 1;
                         }
        | NUMBER_OF_VALUES_X '=' IVAL 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   no_values_x = $3.ival;
			   if (no_values_x < 1) {
			     yyerror("Illegal specification. NUMBER_OF_VALUES_X must be larger or equal 1.");
			   }
			   done = 1;
                         }
	| NUMBER_OF_VALUES_Y '=' IVAL 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   no_values_y = $3.ival;
			   if (no_values_y < 1) {
			     yyerror("Illegal specification. NUMBER_OF_VALUES_Y must be larger or equal 1.");
			   }
			   done = 1;
                         }
	| ALPHA '=' Value 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   alpha = $3.fval;
			   done = 1;
                         }
	| BETA '=' Value 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   beta = $3.fval;
			   if (needbetarange) {
			     yyerror("Illegal specification. Need range specification: beta in [Value_1,Value_2].");
			   }
			   done = 1;
                         }
	| BETA '=' '[' Value ',' Value ']'  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   if ($4.fval < $6.fval) {
				beta_start = $4.fval;	
				beta_end = $6.fval;
			   } else {
				beta_start = $6.fval;
				beta_end   = $4.fval;
			   }
			   if (delta_param == INVALID) {
				delta_param = (beta_end - beta_start)/DEFAULT_NO_PARAMETER;
				delta_param = (delta_param < MIN_DELTA_PARAM) ? MIN_DELTA_PARAM : delta_param;
			   }
		           if (needbetarange == 0) {
			     yyerror("Illegal specification. No range specification allowed here");
			   }
			   needbetarange++;
			   done = 1;
	                }
	| PARAMETER_STEPSIZE '=' Value 
			{  static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   delta_param = $3.fval;
			   if (delta_param < MIN_DELTA_PARAM) {
			     sprintf(strg,"Illegal specification. PARAMETER_STEPSIZE >= %g.",MIN_DELTA_PARAM);
			     yyerror(strg);
			   }
			   done = 1;
                         }
	| SIGMA '=' Value 
			{ static int done=0; 

                           if (done) yyerror("Multiple declaration.");
			   sigma = $3.fval;
			   if ((sigma < MIN_SIGMA) || (sigma > MAX_SIGMA)) {
		      sprintf(strg,"Illegal specification. Admissible range for SIGMA: [%g, %g].",MIN_SIGMA,MAX_SIGMA);
				yyerror(strg);
			   }
			   done = 1;
                         }
	| TIME_TO_FIND_GRID_DIMENSIONS '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   time_to_find_grid_dimensions = fabs($3.fval);
			   done = 1;
	                }
	| TIME_FOR_RELAXATION_OF_TRAJECTORY '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   time_for_relaxation = fabs($3.fval);
			   done = 1;
	                }
	| GRID_FIXED '=' Binary 
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   grid_fixed_flag = (int) $3.ival;
			   done = 1;
	                }
	| TIME_TO_REACH_STATIONARY_SOLUTION '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   t_until_stationary = fabs($3.fval);
			   done = 1;
	                }
        | EXPLODE_INFINITY '=' Value  
			{ static int done=0;

                           if (done) yyerror("Multiple declaration.");
			   if ( ($3.fval > MAX_EXPLODE_INFINITY) || ($3.fval <= 0) ) {
			     sprintf(strg,"Illegal specification.  0 < EXPLODE_INFINITY <=  %g.",
				     MAX_EXPLODE_INFINITY);
			     yyerror(strg);
                           }
			   explode_infinity = fabs($3.fval);
			   done = 1;
	                }
	| PULLBACK_TIMES '=' Value_List 
                         { static int done=0;
			   int        i,j;

	                   if (done) yyerror("Multiple declaration.");
			   /*  value_list: List of increasing nonnegative values */
			   qsort(value_list, list_idx, sizeof(value_list[0]),compare);
			   j=0;
			   for(i=0; i<list_idx; i++) {
			      if (DEBUG) {
                                fprintf(stderr,"PARSER: PB_value_list[%d]: %g\n",i,value_list[i]);
			      }
			      /* Null is removed form the list */
			      if ( value_list[i] !=0 ) {
			        if ( (j==0) || ((j>0) && (value_list[i] != pb_times[j-1])) ) {
			          pb_times[j++] = value_list[i];
			        }
			      }
			   }
			   no_pb_times = j;
			   list_idx = 0;
			   done = 1;
			}
	| SNAPSHOT_TIMES '=' Value_List 
                         { static int  done=0;
			   int         i,j;

	                   if (done) yyerror("Multiple declaration.");
			   /*  value_list: List of increasing nonnegative values */
			   qsort(value_list, list_idx, sizeof(value_list[0]),compare);
			   j=0;
			   for(i=0; i<list_idx; i++) {
			      if (DEBUG) {
			         fprintf(stderr,"PARSER: FB_value_list[%d]: %g\n",i,value_list[i]);
			      }
			      /* Null is removed form the list */
			      if ( value_list[i] !=0 ) {
				if ( (j==0) || ((j>0) && (value_list[i] != snapshot_times[j-1])) ) {
				   snapshot_times[j++] = value_list[i];
				}
			      }
			   }
			   no_snapshot_times = j;
			   list_idx = 0;
			   done = 1;
	                 }

	| FILENAME '=' STRING 
	                     { static int done=0; 

			       if (done) yyerror("Multiple declaration.");
			       if ( (filenamesize=strlen($3.name)) !=0 ) {
				 filename = salloc((filenamesize+FILENAME_POSTFIX_SIZE+2)*sizeof(char));
				 strcpy(filename, $3.name);
			       }
			       done = 1;
                             }

;
/* In Value_List all values are fed in as absolute values!! */

Value_List : Value   { if (list_idx == MAX_LIST_LEN) { 
                         sprintf(strg,"Illegal specification. Allowed maximum number of times is %d. ",MAX_LIST_LEN);
			 yyerror(strg);
                       }
		       else  value_list[list_idx++] = fabs($1.fval); 
		     } 
| Value_List Value   { if (list_idx == MAX_LIST_LEN) {
                         sprintf(strg,"Illegal specification. Allowed maximum number of times is %d. ",MAX_LIST_LEN);
			 yyerror(strg);
                        }
			else  value_list[list_idx++] = fabs($2.fval); 
		      }
;


Value: IVAL { $$.fval = $1.ival; }
     | FVAL { $$.fval = $1.fval; }
;


Binary  : TRUE   { $$.ival = 1; }
	| FALSE  { $$.ival = 0; }
;

%%

int yyerror(char  *s) {

  error++;
  fprintf(stderr, "ERROR(line %d): %s\n", line_nr, s);

  return 0;
}

int main(int argc, char *argv[]) {
  int    i;
  char   path[256], *c, *prgname, zeichen;
  char   call[MAX_LIST_LEN*16+MAX_STRING_SIZE]={0};
  char   call_tmp[MAX_LIST_LEN*16+MAX_STRING_SIZE]={0};
  char   real_string[MAX_STRING_SIZE]={0};

  strcpy(path,argv[0]);
  c = NULL;
#if defined(__MSDOS__) || defined(__WIN32__)
  c = strstr(path,":");
  if (c != NULL) 
    c++;
#endif
  if (c==NULL) 
    c = path;
  
  prgname = c;
  while (*c) {
    if (*c == SLASH)
      prgname = c+1;
    c++;
  }
  *prgname = 0;
  
  if (argc == 2) {
	yyin = fopen(argv[1],"rt");
	if (yyin == NULL) {
	  sprintf(strg,"File <%s> not found.",argv[1]);
	  yyerror(strg);
	  return 1;
	}
  } else {
    sprintf(strg,"Usage: %s filename",argv[0]);
    yyerror(strg);
    return 1;
  }

  yyparse();
  if (error) {
    sprintf(strg,"Syntax error in parameter file %s.",argv[1]);
    yyerror(strg);
    return 1;
  }
  if (needbetarange == 1) {
	yyerror("Missing range specification of BETA.");
	return 1;
  }
  if ( (lx_s==0.0) && (ly_s==0.0) ) {
    yyerror("(IVALUE_LINEARIZED_SYSTEM_X,IVALUE_LINEARIZED_SYSTEM_Y)=(0,0) not allowed.");
    return 1;
  }
  if (no_pb_times == 0) {
    pb_times[0] = 100*dt;
    no_pb_times = 1;
  }
  for (i=0;i<no_pb_times;i++) {
    if (pb_times[i] < dt) {
      sprintf(strg,"Syntax error in parameter file %s. Pullback times must be larger than dt=%g.",
	      argv[1],dt);
      yyerror(strg);
      return 1;  
    }
  }
  if (no_snapshot_times == 0) {
    snapshot_times[0] = 100*dt;
    no_snapshot_times = 1;
  }
  for (i=0;i<no_snapshot_times;i++) {
    if (snapshot_times[i] < dt) {
      sprintf(strg,"Syntax error in parameter file %s. Snapshot times must be larger than dt=%g.",
	      argv[1],dt);
      yyerror(strg);
      return 1;  
    }
  }

  if (filenamesize==0) {
    filename = salloc((strlen(program[procedure])+1)*sizeof(char));
    strcpy(filename,program[procedure]);
    filenamesize = strlen(filename);
  } 
  switch(procedure) {
        case HISTOGRAM_PROC: 
                             sprintf(call,"%s%s %d %s %ld %g %g %g %g  %ld %ld %g %g %g %g %g %g %g %g %g %d",
				    path,program[procedure],
                                    filenamesize,filename,
				    gauss_seed, t_end,
				    dt,x_s,y_s,
				    no_bins_x, no_bins_y,
				    x_low,x_high,y_low,y_high,
				    alpha,beta,sigma,
				    time_to_find_grid_dimensions,time_for_relaxation,
				    grid_fixed_flag);
			break;
        case LYAPUNOV_PROC:  
                             sprintf(call,"%s%s %d %s %ld %g %g %g %g %g %g %g %g %g %g %g %g %g %g",
				    path,program[procedure],
                                    filenamesize,filename,
				    gauss_seed,
				    t_end,dt,lx_s,ly_s,
				    nlx1_s,nly1_s,nlx2_s,nly2_s,
				    alpha,beta_start,beta_end,
				    sigma,delta_param,t_until_stationary);
	                break;
        case PULLBACK_PROC:  
                             sprintf(call,"%s%s %d %s %ld %ld %g %g %g %g %g %ld %ld %g %g %g",
				     path,program[procedure],
				     filenamesize,filename,
				     gauss_seed,
				     no_pb_times,
				     dt,x_low,x_high,y_low,y_high,
				     no_values_x, no_values_y,
				     alpha,beta,sigma);
			     for (i=0;i<no_pb_times;i++) {
			       /* pb_times ist absteigende Liste */
			       sprintf(real_string,"  %g",pb_times[no_pb_times-1-i]);
			       strcat(call_tmp,real_string);
			     }
			    strcat(call,call_tmp);
	                break;
        case FLOW_BWD_PROC:  
                              sprintf(call,"%s%s %d %s %ld %ld %g %g %g %g %g %ld %ld %g %g %g",
				      path,program[procedure],
				      filenamesize,filename,
				      gauss_seed,
				      no_snapshot_times,
				      dt,x_low,x_high,y_low,y_high,
				      no_values_x, no_values_y,
				      alpha,beta,sigma);
                              for (i=0;i<no_snapshot_times;i++) {
				/* snapshot_times ist aufsteigende Liste */
				sprintf(real_string,"  %g",snapshot_times[i]);
				strcat(call_tmp,real_string);
			      }
			      strcat(call,call_tmp);
	                  break;
        case TRAJECTORY_PROC: 
                              sprintf(call,"%s%s %d %s %ld %g %g %g %g %g %g %g",
				      path,program[procedure],
				      filenamesize,filename,
				      gauss_seed,t_end,
				      dt,x_s,y_s,alpha,beta,sigma);
			  break;
        case TRAJECTORY_BWD_PROC:  
                                   sprintf(call,"%s%s %d %s %ld %g %g %g %g %g %g %g",
					   path,program[procedure],
					   filenamesize,filename,
					   gauss_seed,t_end,
					   dt,x_s,y_s,alpha,beta,sigma);
			break;
        case ROTNUMBER_PROC:    
                                  sprintf(call,"%s%s %d %s %ld %g %g %g %g %g %g %g %g %g %g %g %g %g %g",
					  path,program[procedure],
					  filenamesize,filename,
					  gauss_seed,
					  t_end,dt,lx_s,ly_s,
					  nlx1_s,nly1_s,nlx2_s,nly2_s,
					  alpha,beta_start,beta_end,
					  sigma,delta_param,t_until_stationary);
	                break;
        case BACKWARD_EXPLOSION_TIMES_PROC:  
	                                     sprintf(call,"%s%s %d %s %ld %g %g %g %g %g %g %ld %ld %g %g %g %g",
						     path,program[procedure],
						     filenamesize,filename,
						     gauss_seed,t_end,dt,
						     x_low,x_high,y_low,y_high,
						     no_values_x, no_values_y,
						     alpha,beta,sigma,explode_infinity);
	               break;
        case LYAPUNOV_ONED_PROC:  
                             sprintf(call,"%s%s %d %s %ld %g %g %g %g %g %g %g %g %g %g",
				    path,program[procedure],
                                    filenamesize,filename,
				    gauss_seed,t_end,dt,
                                    lx_s,nlx1_s,nlx2_s,
				    beta_start,beta_end,sigma,
                                    delta_param,t_until_stationary);
	                break;
        case BWD_EXPLOSION_TIMES_ONED_PROC:  
                              sprintf(call,"%s%s %d %s %ld %g %g %g %g %ld %g %g %g",
				      path,program[procedure],
				      filenamesize,filename,
				      gauss_seed,t_end,dt,
				      x_low,x_high,no_values_x, 
				      -beta,sigma,explode_infinity);
                       /* -beta, as the algorithm underlies the system backward in time */
	                break;
        case FLOW_BWD_ONED_PROC:  
                              sprintf(call,"%s%s %d %s %ld %ld %g %g %g %ld %g %g",
				      path,program[procedure],
				      filenamesize,filename,
				      gauss_seed,no_snapshot_times,
				      dt,x_low,x_high,no_values_x, 
				      beta,sigma);
                              for (i=0;i<no_snapshot_times;i++) {
				/* snapshot_times ist aufsteigende Liste */
				sprintf(real_string,"  %g",snapshot_times[i]);
				strcat(call_tmp,real_string);
			      }
			      strcat(call,call_tmp);
	                  break;
  } 
  if (argc == 2) fclose(yyin);

  if (DEBUG) {
    fprintf(stderr,"%s\n",call); fflush(stderr); 
    fprintf(stderr,"Parser: no_pb_times: %ld\n",no_pb_times);
    for(i=0; i<no_pb_times; i++) {
      fprintf(stderr,"Parser: PB_times[%d]: %g\n",i,pb_times[i]);
    }
    fprintf(stderr,"Parser: no_snapshot_times: %ld\n",no_snapshot_times);
    for(i=0; i<no_snapshot_times; i++) {
      fprintf(stderr,"Parser: snapshot_times[%d]: %g\n",i,snapshot_times[i]);
    }
    fprintf(stderr,"ENDE PARSER\n");fflush(stderr);
  }

  putenv("BRUSPARS=jhgtgrteukz");
  i = system(call); 
  putenv("BRUSPARS=");

  strcat(filename,".log");
  fp_log = fopen(filename,"rt");
  if (fp_log != NULL) {
    while ((zeichen=fgetc(fp_log)) != EOF) 
      fputc(zeichen,stdout);
    fclose(fp_log);
  }

  return i;
}

