%{
#include <stdlib.h>
#include <string.h>

#include "stype.h"
#include "bparser.h"

void* salloc(size_t i);
extern FILE  *yyin;
extern int yyerror(char *s);

int line_nr=1;

%}

D                       [0-9]
E                       [Ee][+-]?{D}+
S                       [+-]



%%


"PROCEDURE"              { return PROCEDURE;}
"SEED"                   { return SEED; }
"T_END"                  { return T_END; }
"TIME_STEPSIZE"          { return TIME_STEPSIZE; }
"INITIAL_VALUE_X"        { return INITIAL_VALUE_X; }
"INITIAL_VALUE_Y"        { return INITIAL_VALUE_Y; }
"INITIAL_VALUE_FOR_LINEARIZED_SYSTEM_X"  { return IVALUE_LINEARIZED_SYSTEM_X; }
"INITIAL_VALUE_FOR_LINEARIZED_SYSTEM_Y"  { return IVALUE_LINEARIZED_SYSTEM_Y; }
"INITIAL_VALUE_FOR_NONLINEAR_SYSTEM_X1"  { return IVALUE_NONLINEAR_SYSTEM_X1; }
"INITIAL_VALUE_FOR_NONLINEAR_SYSTEM_X2"  { return IVALUE_NONLINEAR_SYSTEM_X2; }
"INITIAL_VALUE_FOR_NONLINEAR_SYSTEM_Y1"  { return IVALUE_NONLINEAR_SYSTEM_Y1; }
"INITIAL_VALUE_FOR_NONLINEAR_SYSTEM_Y2"  { return IVALUE_NONLINEAR_SYSTEM_Y2; }
"X_RANGE"                { return X_RANGE; }
"Y_RANGE"                { return Y_RANGE; }
"NUMBER_OF_BINS_X"       { return NUMBER_OF_BINS_X; }
"NUMBER_OF_BINS_Y"       { return NUMBER_OF_BINS_Y; }
"NUMBER_OF_VALUES_X"     { return NUMBER_OF_VALUES_X; }
"NUMBER_OF_VALUES_Y"     { return NUMBER_OF_VALUES_Y; }
"ALPHA"                  { return ALPHA; }
"BETA"                   { return BETA; }
"SIGMA"                  { return SIGMA; }
"PARAMETER_STEPSIZE"     { return PARAMETER_STEPSIZE;}
"TIME_TO_FIND_GRID_DIMENSIONS"      { return TIME_TO_FIND_GRID_DIMENSIONS; }
"TIME_FOR_RELAXATION_OF_TRAJECTORY" { return TIME_FOR_RELAXATION_OF_TRAJECTORY; }
"TIME_TO_REACH_STATIONARY_SOLUTION" { return TIME_TO_REACH_STATIONARY_SOLUTION; }
"GRID_FIXED"             { return GRID_FIXED; }
"PULLBACK_TIMES"         { return PULLBACK_TIMES; }
"SNAPSHOT_TIMES"         { return SNAPSHOT_TIMES; }
"FILENAME"               { return FILENAME; }
"EXPLODE_INFINITY"       { return EXPLODE_INFINITY; }


"TRUE"                      { return TRUE;}
"FALSE"                     { return FALSE;}

"HISTOGRAM"                     { return HISTOGRAM; }
"LYAPUNOV_EXPONENT"             { return LYAPUNOV; }
"TRAJECTORY"                    { return TRAJECTORY; }
"TRAJECTORY_BWD"                { return TRAJECTORY_BWD; }
"ROTATION_NUMBER"               { return ROTNUMBER; }
"PULLBACK"                      { return PULLBACK; }
"FLOW_BWD"                      { return FLOW_BWD; }
"BACKWARD_EXPLOSION_TIMES"      { return BACKWARD_EXPLOSION_TIMES; }
"LYAPUNOV_EXPONENT_1D"          { return LYAPUNOV_ONED; }
"BACKWARD_EXPLOSION_TIMES_1D"   { return BWD_EXPLOSION_TIMES_ONED; }
"FLOW_BWD_1D"                   { return FLOW_BWD_ONED; }

({S})?{D}+               { yylval.ival = atol(yytext);
			   return IVAL; }

({S})?{D}+{E}            { yylval.fval = atof(yytext); return FVAL; }
({S})?{D}*"."{D}+({E})?  { yylval.fval = atof(yytext); return FVAL; }
({S})?{D}+"."{D}*({E})?  { yylval.fval = atof(yytext); return FVAL; }


\"(\\.|[^\ \t\v\f\n\\"])*\" {   char *c;

                           /*   " at the begin and end of string is deleted */ 
			   yylval.name = salloc((strlen(yytext)+1)*sizeof(char));
			   strcpy(yylval.name, yytext+1);
			   c = yylval.name+strlen(yytext)-2;
			   *c = 0;
			   return STRING; }

"["                     { return '['; }
"]"                     { return ']'; }
";"                     { return ';'; }
","			{ return ','; }
"="			{ return '='; }

[ \t\v\n\f]             { /* ignore  white spaces */ 
			  if (*yytext == '\n')
				line_nr++;
			}

"#".*                   { /* ignore comments */ }

.                       { /* ignore  stupid characters */ 
			  fprintf(stderr,"WARNING(line %d): character (%s).\n",
					line_nr,yytext);
			}

%%
#if defined(__MSDOS__) || defined(__WIN32__)
/* nothing */
#else
int yywrap() 
{
        return(1);
}
#endif

void* salloc(size_t i) {
	void *p;

	p = malloc(i);
	if (p == NULL) {
		yyerror("Not enough memory");
		exit(1);
	}

	return p;
}





























