#include <stdlib.h>
#include <stdio.h> 
#include <string.h>
#include <math.h>
#include <values.h>

#include "utils.h"
#include "j.h"
#include "brus_slv.h"

/***  global variables  ***/

long            gauss_seed;
unsigned long   no_values_x,no_values_y; 
double          alpha, beta, sigma;
unsigned long   iterations;
double          dt,t_end,d_initial;                    
double          x_low,x_high,y_low,y_high;
double          *xnon, *ynon, *xnon_initial, *ynon_initial;
double          *x_coord, *y_coord, *d;
char            *user_filename;
int             user_filenamesize;
double          explode_infinity;

/***  global variables  ***/

void initialize(char *argv[]){

  unsigned long   i,j,no_initial_points;
  double          dx,dy;
  
  sscanf(argv[1],   "%d",    &user_filenamesize);
  user_filename = salloc((user_filenamesize+1)*sizeof(*user_filename));
  sscanf(argv[2],   "%s",    user_filename);
  sscanf(argv[3],   "%ld",   &gauss_seed);
  sscanf(argv[4],   "%lf",   &t_end); 
  sscanf(argv[5],   "%lf",   &dt);
  sscanf(argv[6],   "%lf",   &x_low); 
  sscanf(argv[7],   "%lf",   &x_high);
  sscanf(argv[8],   "%lf",   &y_low); 
  sscanf(argv[9],   "%lf",   &y_high);
  sscanf(argv[10],  "%lu",   &no_values_x);
  sscanf(argv[11],  "%lu",   &no_values_y);
  sscanf(argv[12],  "%lf",   &alpha);
  sscanf(argv[13],  "%lf",   &beta);
  sscanf(argv[14],  "%lf",   &sigma);
  sscanf(argv[15],  "%lf",   &explode_infinity);
  no_initial_points = (no_values_x+1) * (no_values_y+1);
  iterations = (unsigned long) round(t_end / dt);
  if (DEBUG) {
    fprintf(stderr,"\nArguments for <%s>\n\n",argv[0]);
    fprintf(stderr,  "user_filenamesize: %8d\n",    user_filenamesize);
    fprintf(stderr,  "user_filename:     <%s>\n",   user_filename);
    fprintf(stderr,  "gauss_seed=        %8ld\n", gauss_seed);
    fprintf(stderr,  "t_end=             %8g\n",  t_end); 
    fprintf(stderr,  "dt=                %8g\n",  dt);
    fprintf(stderr,  "x_low=             %8g\n",  x_low); 
    fprintf(stderr,  "x_high=            %8g\n",  x_high);
    fprintf(stderr,  "y_low=             %8g\n",  y_low); 
    fprintf(stderr,  "y_high=            %8g\n",  y_high);
    fprintf(stderr,  "no_values_x=       %8lu\n",  no_values_x);
    fprintf(stderr,  "no_values_y=       %8lu\n",  no_values_y);
    fprintf(stderr,  "alpha=             %8g\n",  alpha);
    fprintf(stderr,  "beta=              %8g\n",  beta);
    fprintf(stderr,  "sigma=             %8g\n",  sigma);
    fprintf(stderr,  "explode_infinity=  %8g\n",  explode_infinity);
    fprintf(stderr,  "iterations:        %8lu\n", iterations);
  }
  xnon            = (double*) salloc(no_initial_points*sizeof(*xnon));
  xnon_initial    = (double*) salloc(no_initial_points*sizeof(*xnon_initial));
  ynon            = (double*) salloc(no_initial_points*sizeof(*ynon));
  ynon_initial    = (double*) salloc(no_initial_points*sizeof(*ynon_initial));
  x_coord = (double*) salloc((no_values_x+1)*sizeof(*x_coord));
  y_coord = (double*) salloc((no_values_y+1)*sizeof(*y_coord));
  d       = (double*) salloc(no_initial_points*sizeof(*d));
  dx = (x_high - x_low) / ((double) no_values_x);
  dy = (y_high - y_low) / ((double) no_values_y);
  d_initial = -(2*t_end);
  for (i=0; i<=no_values_x; i++){
    x_coord[i] = x_low + ((double) i) * dx;
  }
  for (j=0; j<=no_values_y; j++){
    y_coord[j] = y_low + ((double) j) * dy;
  }
  /* 
     Zeilenweises Abspeichern der x-Koordinaten Matrizen "xnon[0:no_values_y][0:no_values_x]" 
     und y-Koordinaten Matrizen "ynon[0:no_values_y][0:no_values_x]" 
  */
  for (i=0; i<=no_values_x; i++){
    for (j=0; j<=no_values_y; j++){
      xnon[j*(no_values_x+1)+i]         = x_low + ((double) i) * dx;
      xnon_initial[j*(no_values_x+1)+i] = x_low + ((double) i) * dx;
      ynon[j*(no_values_x+1)+i]         = y_low + ((double) j) * dy; 
      ynon_initial[j*(no_values_x+1)+i] = y_low + ((double) j) * dy; 
      d[j*(no_values_x+1)+i] = d_initial;
    }
  }
}


/* 
   Apropos Matrizen-Abspeichern fuer matlab:

   dvector = salloc(no_x * no_y * sizeof(*dvector));
   for(i=0; i<no_x; i++) {
     for(j=0; j<no_y; j++) {
       dvector[i*no_y+j] =  d[i][j];
     }
   }
   savemat(f,TYPE_FLAG,"d",no_y,no_x,0,dvector, (double *) NULL); 
   savemat(f,TYPE_FLAG,"xvec",1,no_x,0,xvector, (double *) NULL); 
   savemat(f,TYPE_FLAG,"yvec",1,no_y,0,yvector, (double *) NULL); 

	Zeilenweises Abspeichern einer Matrix 
	hat in matlab die Transponierte zum Ergebnis:

	"load mat_test.mat" in matlab hat zum Ergebnis:

	d[j][i] also die Transponierte von d!!!

	Aufruf von mesh waere hier:

        denn mesh(x(j),y(i),Z(i,j)), wobei x zu den Spalten von Z korrespondiert und
                                           y zu den Zeilen  von Z korrespondiert 
                                           
        mit size(Z) = (m,n), length(x) = n, length(y) = m, 
        also bei mesh muss (Spalte,Zeile,Z(Zeile, Spalte)) spezifiert werden!!!
                            ------------

	load mat_test.mat
	figure(1)
	x1 = xvec
	y1 = yvec
	d1= d
	mesh(y1,x1,d1')     (Z bezeichnet die Transponierte einer Matrix)
	xlabel('y')
	ylabel('x')
*/

void save_data_in_matlab_format(char  *filename_mat) {

  FILE    *fp;
  double  tmp;

  if ((fp = fopen(filename_mat,"wb")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n", filename_mat);fflush(stderr);
    exit(1);  
  }
  tmp= log10((double) t_end);
  savemat(fp,TYPE_FLAG,"log10_t_end",         1,1,0,                          &tmp,         (double *) NULL);
  tmp = log10((double) explode_infinity);
  savemat(fp,TYPE_FLAG,"log10_explode_infty", 1,1,0,                          &tmp,         (double *) NULL);
  
  /* bei zeilenweisem Abspeichern der Matrix zuerst die Zeilenlaenge angeben:
     hier Zeilenlaenge = no_values_x+1 */

  savemat(fp,TYPE_FLAG, "bwd_expl_times_M",   no_values_x+1, no_values_y+1,0, d,            (double *) NULL); 
  savemat(fp,TYPE_FLAG, "initial_values_x_M", no_values_x+1, no_values_y+1,0, xnon_initial, (double *) NULL); 
  savemat(fp,TYPE_FLAG, "initial_values_y_M", no_values_x+1, no_values_y+1,0, ynon_initial, (double *) NULL); 
  savemat(fp,TYPE_FLAG, "x_at_expl_time_M",   no_values_x+1, no_values_y+1,0, xnon,         (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_at_expl_time_M",   no_values_x+1, no_values_y+1,0, ynon,         (double *) NULL); 
  savemat(fp,TYPE_FLAG, "x_coordinates_V",    1,no_values_x+1,0,              x_coord,      (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_coordinates_V",    1,no_values_y+1,0,              y_coord,      (double *) NULL); 
  savemat(fp,TYPE_FLAG, "explodetime_infty",  1,1,0,                          &d_initial,   (double *) NULL);
  savemat(fp,TYPE_FLAG, "alpha",              1,1,0,                          &alpha,       (double *) NULL);
  savemat(fp,TYPE_FLAG, "beta",               1,1,0,                          &beta,        (double *) NULL);
  savemat(fp,TYPE_FLAG, "sigma",              1,1,0,                          &sigma,       (double *) NULL); 
  savemat(fp,TYPE_FLAG, "time_stepsize",      1,1,0,                          &dt,          (double *) NULL);
  savemat(fp,TYPE_FLAG, "x_low",              1,1,0,                          &x_low,       (double *) NULL); 
  savemat(fp,TYPE_FLAG, "x_high",             1,1,0,                          &x_high,      (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_low",              1,1,0,                          &y_low,       (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_high",             1,1,0,                          &y_high,      (double *) NULL); 
  tmp = (double ) no_values_x;
  savemat(fp,TYPE_FLAG, "number_of_values_x", 1,1,0,                          &tmp,         (double *) NULL); 
  tmp = (double ) no_values_y;
  savemat(fp,TYPE_FLAG, "number_of_values_y", 1,1,0,                          &tmp,         (double *) NULL); 
  tmp = (double ) gauss_seed;
  savemat(fp,TYPE_FLAG, "seed",               1,1,0,                          &tmp,         (double *) NULL); 
  fclose(fp);
}

void save_data_in_ascii_format(char  *filename_asc) {
  
  FILE           *fp;
  unsigned long  i,j;
  double         tmp;


  if ((fp = fopen(filename_asc,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n",filename_asc);fflush(stderr);
    exit(1);  
  }
  fprintf(fp,  "## Parameters:\n##\n");
  fprintf(fp, "#alpha=                   %8g\n",   alpha);
  fprintf(fp, "#beta=                    %8g\n",   beta);
  fprintf(fp, "#sigma=                   %8g\n",   sigma);
  fprintf(fp, "#time_stepsize=           %8g\n",   dt);
  fprintf(fp, "#t_end=                   %8g\n",   t_end);
  fprintf(fp, "#x_low=                   %8g\n",   x_low); 
  fprintf(fp, "#x_high=                  %8g\n",   x_high); 
  fprintf(fp, "#y_low=                   %8g\n",   y_low); 
  fprintf(fp, "#y_high=                  %8g\n",   y_high); 
  fprintf(fp, "#number_of_values_x=      %8lu\n",  no_values_x);
  fprintf(fp, "#number_of_values_y=      %8lu\n",  no_values_y);
  fprintf(fp, "#seed=                    %8ld\n",  gauss_seed);
  fprintf(fp, "#explodetime_infinity=    %8g\n",   d_initial);
  tmp = log10((double) explode_infinity);
  fprintf(fp, "#log10_explode_infinity=  %8g\n",   tmp);
  fprintf(fp,"\n\n\n");
  fprintf(fp,"## Backward explosion times (BEt= backward explosion time):\n##\n");
  fprintf(fp,"##%6s\t%8s\t%8s\t",
	  "x(0)", "y(0)", "BEt");
  fprintf(fp,"%8s\t%8s\n\n",
	  "x(BEt)", "y(BEt)");
  for (j=0; j<=no_values_y; j++){
    for (i=0; i<=no_values_x; i++){
      fprintf(fp,"%8g\t%8g\t%8g\t%8g\t%8g\n",
	      xnon_initial[j*(no_values_x+1)+i],
	      ynon_initial[j*(no_values_x+1)+i],
	      d[j*(no_values_x+1)+i],
	      xnon[j*(no_values_x+1)+i],
	      ynon[j*(no_values_x+1)+i]);
    }
    fprintf(fp,"\n");
  }
  fclose(fp);
}

int main(int argc, char *argv[]) {

  unsigned long    i,k,no_initial_points;
  double           current_time;
  double           sde_vec[SDE_DIMENSION];
  char             filename_mat[NAMESIZE],filename_asc[NAMESIZE];
  char             filename_log[NAMESIZE];
  FILE             *fp_log;
  
  testenv(argv[0]);
  
  initialize(argv);
  sprintf(filename_mat, "%s.mat", user_filename);
  sprintf(filename_asc, "%s.dat", user_filename);
  sprintf(filename_log, "%s.log", user_filename);
  if ((fp_log = fopen(filename_log,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n",filename_log);
    exit(1);  
  }
  no_initial_points = (no_values_x+1)*(no_values_y+1);

  gauss_init(gauss_seed);
  current_time = 0.0;
  for(k=0;k<iterations;k++) {
    fill_gauss(&gauss1);
    compute_j_functionals(&gauss1,dt);
    for(i=0; i<no_initial_points; i++) {
      sde_vec[0] = xnon[i];
      sde_vec[1] = ynon[i];
      if (d[i] == d_initial) {
	if ( norm_euklid_sqr(sde_vec, SDE_DIMENSION)>= explode_infinity ) {
	  d[i] = -current_time;
	} 
	else {
	  Brus_nonlinear_ord2_bwd_for_grid (&(xnon[i]),&(ynon[i]),dt,
					    alpha,beta,sigma);
	}
      }
    } 
    current_time += dt;
  }
  save_data_in_matlab_format(filename_mat); 
  save_data_in_ascii_format(filename_asc); 
  fclose(fp_log);

  return 0;
}








