#include <stdlib.h>
#include <stdio.h> 
#include <string.h>
#include <math.h>
#include <values.h>

#include "utils.h"
#include "j.h"
#include "oned_slv.h"

/******  global variables  ******/

long            gauss_seed;
unsigned long   no_values_x;
double          beta, sigma;
unsigned long   iterations;
double          dt, t_end, d_initial;                    
double          x_low, x_high;
double          *xnon, *xnon_initial, *x_coord, *d;
char            *user_filename;
int             user_filenamesize;
double          explode_infinity;

/******  global variables  ******/

void initialize (char *argv[]) {
  
  unsigned long   i,no_initial_points;
  double          dx=0.0;
  
  sscanf(argv[1],   "%d",   &user_filenamesize);
  user_filename = salloc((user_filenamesize+1)*sizeof(*user_filename));
  sscanf(argv[2],   "%s",   user_filename);
  sscanf(argv[3],   "%ld",  &gauss_seed);
  sscanf(argv[4],   "%lf",  &t_end); 
  sscanf(argv[5],   "%lf",  &dt);
  sscanf(argv[6],   "%lf",  &x_low); 
  sscanf(argv[7],   "%lf",  &x_high);
  sscanf(argv[8],   "%lu",  &no_values_x);
  sscanf(argv[9],   "%lf",  &beta);
  sscanf(argv[10],  "%lf",  &sigma);
  sscanf(argv[11],  "%lf",  &explode_infinity);
  no_initial_points = no_values_x + 1;
  iterations = (unsigned long) round(t_end / dt);
  if (DEBUG) {
    fprintf(stderr,  "\nArguments for <%s>\n\n",  argv[0]);
    fprintf(stderr,  "user_filenamesize: %8d\n",  user_filenamesize);
    fprintf(stderr,  "user_filename:     <%s>\n", user_filename);
    fprintf(stderr,  "gauss_seed=        %8ld\n", gauss_seed);
    fprintf(stderr,  "t_end=             %8g\n",  t_end); 
    fprintf(stderr,  "dt=                %8g\n",  dt);
    fprintf(stderr,  "x_low=             %8g\n",  x_low); 
    fprintf(stderr,  "x_high=            %8g\n",  x_high);
    fprintf(stderr,  "no_values_x=       %8lu\n", no_values_x);
    fprintf(stderr,  "beta=              %8g\n",  beta);
    fprintf(stderr,  "sigma=             %8g\n",  sigma);
    fprintf(stderr,  "explode_infinity=  %8g\n",  explode_infinity);
    fprintf(stderr,  "iterations=        %8lu\n", iterations);
  }
  xnon            = (double*) salloc(no_initial_points*sizeof(*xnon));
  xnon_initial    = (double*) salloc(no_initial_points*sizeof(*xnon_initial));
  x_coord = (double*) salloc(no_initial_points*sizeof(*x_coord));
  d       = (double*) salloc(no_initial_points*sizeof(*d));
  dx = (x_high - x_low) / ((double) no_values_x);
  d_initial = -(2*t_end);
  for (i=0;i<no_initial_points;i++){
    xnon[i] = x_coord[i] = x_low + ((double) i) * dx;
    d[i] = d_initial;
  }
}

void save_data_in_matlab_format(char  *filename_mat) {

  FILE            *fp;
  double          tmp;
  unsigned long   no_initial_points = no_values_x+1;

  if ((fp = fopen(filename_mat,"wb")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n", filename_mat);fflush(stderr);
    exit(1);  
  }
  tmp= log10((double) t_end);
  savemat(fp,TYPE_FLAG, "log10_t_end",         1,1,0,                   &tmp,       (double *) NULL);
  tmp = log10((double) explode_infinity);
  savemat(fp,TYPE_FLAG, "log10_explode_infty", 1,1,0,                   &tmp,       (double *) NULL);
  savemat(fp,TYPE_FLAG, "bwd_expl_times_V",    1, no_initial_points, 0, d,          (double *) NULL);  
  savemat(fp,TYPE_FLAG, "initial_values_x_V",  1, no_initial_points, 0, x_coord,    (double *) NULL);  
  savemat(fp,TYPE_FLAG, "x_at_expl_time_V",    1, no_initial_points, 0, xnon,       (double *) NULL);  
  savemat(fp,TYPE_FLAG, "x_coordinates_V",     1, no_initial_points, 0, x_coord,    (double *) NULL);  
  savemat(fp,TYPE_FLAG, "explodetime_infty",   1,1,0,                   &d_initial, (double *) NULL);
  savemat(fp,TYPE_FLAG, "beta",                1,1,0,                   &beta,      (double *) NULL);
  savemat(fp,TYPE_FLAG, "sigma",               1,1,0,                   &sigma,     (double *) NULL); 
  savemat(fp,TYPE_FLAG, "time_stepsize",       1,1,0,                   &dt,        (double *) NULL);
  savemat(fp,TYPE_FLAG, "x_low",               1,1,0,                   &x_low,     (double *) NULL); 
  savemat(fp,TYPE_FLAG, "x_high",              1,1,0,                   &x_high,    (double *) NULL); 
  tmp = (double ) no_values_x;
  savemat(fp,TYPE_FLAG, "number_of_values_x",  1,1,0,                   &tmp,       (double *) NULL); 
  tmp = (double ) gauss_seed;
  savemat(fp,TYPE_FLAG, "seed",                1,1,0,                   &tmp,       (double *) NULL); 
  fclose(fp);
}

void save_data_in_ascii_format(char  *filename_asc) {
  
  FILE            *fp;
  unsigned long   i;
  double          tmp;
  unsigned long   no_initial_points = no_values_x+1;


  if ((fp = fopen(filename_asc,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n",filename_asc);
    exit(1);  
  }
  fprintf(fp, "## Parameters:\n#\n");
  fprintf(fp, "#beta=                    %8g\n",   beta);
  fprintf(fp, "#sigma=                   %8g\n",   sigma);
  fprintf(fp, "#time_stepsize=           %8g\n",   dt);
  fprintf(fp, "#t_end=                   %8g\n",   t_end);
  fprintf(fp, "#x_low=                   %8g\n",   x_low); 
  fprintf(fp, "#x_high=                  %8g\n",   x_high); 
  fprintf(fp, "#number_of_values_x=      %8lu\n",  no_values_x);
  fprintf(fp, "#seed=                    %8ld\n",  gauss_seed);
  fprintf(fp, "#explodetime_infinity=    %8g\n",   d_initial);
  tmp = log10((double) explode_infinity);
  fprintf(fp, "#log10_explode_infinity=  %8g\n",   tmp);
  fprintf(fp,"\n\n\n");
  fprintf(fp,"## Backward explosion times (BEt= backward explosion time):\n#\n");
  fprintf(fp,"##%6s\t%8s\t%8s\n\n", "x(0)", "BEt", "x(BEt)");
  for (i=0; i<no_initial_points; i++){
    fprintf(fp,"%8g\t%8g\t%8g\n", x_coord[i], d[i], xnon[i]);
  }
  fclose(fp);
}

int main(int argc, char *argv[])
{
  unsigned long   i,k,no_initial_points;
  double          current_time;
  char            filename_mat[NAMESIZE],filename_asc[NAMESIZE];
  char            filename_log[NAMESIZE];
  FILE            *fp_log;
  
  testenv(argv[0]);
  
  initialize(argv);
  sprintf(filename_mat, "%s.mat", user_filename);
  sprintf(filename_asc, "%s.dat", user_filename);
  sprintf(filename_log, "%s.log", user_filename);
  if ((fp_log = fopen(filename_log,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n",filename_log);fflush(stderr);
    exit(1);  
  }
  no_initial_points = no_values_x+1;

  gauss_init(gauss_seed);
  current_time = 0.0;
  for(k=0;k<iterations;k++) {
    fill_gauss(&gauss1);
    compute_j_functionals_oned(&gauss1,dt);
    for(i=0;i<no_initial_points;i++) {
      if (d[i] == d_initial) {
	if ( fabs(xnon[i]) >= explode_infinity ) {
	  d[i] = -current_time;
	} 
	else {
	  oned_nonlinear_ord2_for_grid (&(xnon[i]),dt,beta,sigma);
	}
      }
    } 
    current_time += dt;
  }
  save_data_in_matlab_format(filename_mat); 
  save_data_in_ascii_format(filename_asc);
  fclose(fp_log);

  return 0;
}









