#include <stdlib.h>
#include <stdio.h> 
#include <string.h>
#include <math.h>
#include <values.h>

#include "utils.h"
#include "j.h"
#include "brus_slv.h"

/***  global variables  ***/

long             gauss_seed;
unsigned long    *snapshot_iterations;
double           *snapshot_times;
unsigned long    no_values_x,no_values_y,no_snapshots; 
double           alpha, beta, sigma;
double           dt;                    
double           x_low,x_high,y_low,y_high;
double           **xnon, **ynon;
char             *user_filename;
int              user_filenamesize;

/***  global variables  ***/

void initialize(char *argv[]) {
  
  unsigned long    i,j,no_initial_points;
  double           dx,dy;

  sscanf(argv[1],   "%d",    &user_filenamesize);
  user_filename = salloc((user_filenamesize+1)*sizeof(*user_filename));
  sscanf(argv[2],   "%s",    user_filename);
  sscanf(argv[3],   "%ld",   &gauss_seed);
  sscanf(argv[4],   "%lu",   &no_snapshots);
  sscanf(argv[5],   "%lf",   &dt);
  sscanf(argv[6],   "%lf",   &x_low); 
  sscanf(argv[7],   "%lf",   &x_high);
  sscanf(argv[8],   "%lf",   &y_low); 
  sscanf(argv[9],   "%lf",   &y_high);
  sscanf(argv[10],  "%lu",   &no_values_x);
  sscanf(argv[11],  "%lu",   &no_values_y);
  sscanf(argv[12],  "%lf",   &alpha);
  sscanf(argv[13],  "%lf",   &beta);
  sscanf(argv[14],  "%lf",   &sigma);
  no_initial_points = (no_values_x+1) * (no_values_y+1);
  no_snapshots++; /* Initial grid is always saved */
  snapshot_times      = salloc(no_snapshots*sizeof(*snapshot_times));
  snapshot_iterations = salloc(no_snapshots*sizeof(*snapshot_iterations));
  snapshot_times[0]      = 0; /* Initial grid */
  snapshot_iterations[0] = 0; /* Initial grid */
  for (i=1;i<no_snapshots;i++) {
    sscanf(argv[14+i],"%lf",&(snapshot_times[i]));
    /* snapshot_iterations[i]: no_timesteps between snapshot[i-1] and snapshot[i] */
    snapshot_iterations[i] = 
      (unsigned long) round((snapshot_times[i]-snapshot_times[i-1]) / dt);
  }
  if (DEBUG) {
    fprintf(stderr,  "\nArguments for <%s>\n\n",argv[0]);
    fprintf(stderr,  "user_filenamesize:   %8d\n",    user_filenamesize);
    fprintf(stderr,  "user_filename:       <%s>\n",   user_filename);
    fprintf(stderr,  "alpha:               %8g\n",    alpha);
    fprintf(stderr,  "beta:                %8g\n",    beta);
    fprintf(stderr,  "sigma:               %8g\n",    sigma);
    fprintf(stderr,  "dt:                  %8g\n",    dt);
    fprintf(stderr,  "x_low:               %8g\n",    x_low); 
    fprintf(stderr,  "x_high:              %8g\n",    x_high);
    fprintf(stderr,  "y_low:               %8g\n",    y_low); 
    fprintf(stderr,  "y_high:              %8g\n",    y_high);
    fprintf(stderr,  "no_values_x:         %8lu\n",   no_values_x);
    fprintf(stderr,  "no_values_y          %8lu\n",   no_values_y);
    fprintf(stderr,  "seed:                %8ld\n",   gauss_seed);
    fprintf(stderr,  "no_snapshots:        %8lu\n",   no_snapshots);
    for (i=0;i<no_snapshots;i++) {
      fprintf(stderr,"snapshot_times[%lu]:      %g\n", i, snapshot_times[i]);
      fprintf(stderr,"snapshot_iterations[%lu]: %lu\n", i, snapshot_iterations[i]);
    }
  }
  xnon = (double **) salloc(no_initial_points*sizeof(*xnon));
  ynon = (double **) salloc(no_initial_points*sizeof(*ynon));
  for (i=0; i<no_initial_points; i++) {
    xnon[i] = (double *) salloc(no_snapshots*sizeof(**xnon));
    ynon[i] = (double *) salloc(no_snapshots*sizeof(**ynon));
  }  /* xynon[no_initial_points][no_snapshots] */
  dx = (x_high - x_low) / ((double) no_values_x);
  dy = (y_high - y_low) / ((double) no_values_y);
  for (i=0; i<=no_values_x; i++) {
    for (j=0; j<=no_values_y; j++) {
      xnon[j*(no_values_x+1)+i][0] = x_low + ((double) i) * dx;
      ynon[j*(no_values_x+1)+i][0] = y_low + ((double) j) * dy;
    }
  }
}

void save_data_in_matlab_format(char  *filename_mat) {

  double          *tmparray;
  FILE            *fp_mat;
  unsigned long   i,j;
  double          tmp;
  unsigned long   no_initial_points=(no_values_x+1)*(no_values_y+1);

  tmparray =  (double *) salloc(no_initial_points*no_snapshots*sizeof(*tmparray));

  if ((fp_mat = fopen(filename_mat,"wb")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n", filename_mat);fflush(stderr);
    exit(1); 
  }
  for (i=0; i<no_initial_points; i++) {
    for (j=0; j<no_snapshots; j++){
      tmparray[i*(no_snapshots)+j] = xnon[i][j];
    }
  }
  savemat(fp_mat,TYPE_FLAG, "flow_x_M", no_snapshots,no_initial_points,0, tmparray,   (double *) NULL); 
  for (i=0; i<no_initial_points; i++) {
    for (j=0; j<no_snapshots; j++){
      tmparray[i*(no_snapshots)+j] = ynon[i][j];
    } 
  }
  savemat(fp_mat,TYPE_FLAG, "flow_y_M", no_snapshots,no_initial_points,0, tmparray,   (double *) NULL); 
  free(tmparray);
  savemat(fp_mat,TYPE_FLAG, "alpha",                1,1,0,            &alpha,         (double *) NULL);
  savemat(fp_mat,TYPE_FLAG, "beta",                 1,1,0,            &beta,          (double *) NULL);
  savemat(fp_mat,TYPE_FLAG, "sigma",                1,1,0,            &sigma,         (double *) NULL); 
  savemat(fp_mat,TYPE_FLAG, "time_stepsize",        1,1,0,            &dt,            (double *) NULL);
  savemat(fp_mat,TYPE_FLAG, "x_low",                1,1,0,            &x_low,         (double *) NULL);
  savemat(fp_mat,TYPE_FLAG, "x_high",               1,1,0,            &x_high,        (double *) NULL);
  savemat(fp_mat,TYPE_FLAG, "y_low",                1,1,0,            &y_low,         (double *) NULL);
  savemat(fp_mat,TYPE_FLAG, "y_high",               1,1,0,            &y_high,        (double *) NULL);
  tmp = (double ) no_values_x;
  savemat(fp_mat,TYPE_FLAG, "number_of_values_x",   1,1,0,            &tmp,           (double *) NULL); 
  tmp = (double ) no_values_y;
  savemat(fp_mat,TYPE_FLAG, "number_of_values_y",   1,1,0,            &tmp,           (double *) NULL); 
  tmp = (double ) gauss_seed;
  savemat(fp_mat,TYPE_FLAG, "seed",                 1,1,0,            &tmp,           (double *) NULL); 
  tmp = (double) no_snapshots;
  savemat(fp_mat,TYPE_FLAG, "number_of_snapshots",  1,1,0,            &tmp,           (double *) NULL); 
  savemat(fp_mat,TYPE_FLAG, "snapshot_times_V",     1,no_snapshots,0, snapshot_times, (double *) NULL); 
  fclose(fp_mat);
}

void save_data_in_ascii_format(char  *filename_asc) {

  FILE           *fp_asc;
  unsigned long  i,j;
  unsigned long  no_initial_points=(no_values_x+1)*(no_values_y+1);


  if ((fp_asc = fopen(filename_asc,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n", filename_asc);fflush(stderr);
    exit(1); 
  }
  fprintf(fp_asc,  "## Parameters:\n##\n");
  fprintf(fp_asc,  "#alpha=                    %8g\n",    alpha);
  fprintf(fp_asc,  "#beta=                     %8g\n",    beta);
  fprintf(fp_asc,  "#sigma=                    %8g\n",    sigma);
  fprintf(fp_asc,  "#time_stepsize=            %8g\n",    dt);
  fprintf(fp_asc,  "#x_low=                    %8g\n",    x_low); 
  fprintf(fp_asc,  "#x_high=                   %8g\n",    x_high); 
  fprintf(fp_asc,  "#y_low=                    %8g\n",    y_low); 
  fprintf(fp_asc,  "#y_high=                   %8g\n",    y_high); 
  fprintf(fp_asc,  "#number_of_values_x=       %8lu\n",   no_values_x);
  fprintf(fp_asc,  "#number_of_values_y=       %8lu\n",   no_values_y);
  fprintf(fp_asc,  "#seed=                     %8ld\n",   gauss_seed);
  fprintf(fp_asc,  "#number_of_snapshots=      %8lu\n",   no_snapshots);
  for (i=0;i<no_snapshots;i++) {
    fprintf(fp_asc,  "#snapshot_times[%lu]=      %8g\n", i+1, snapshot_times[i]);
  }

  fprintf(fp_asc,"\n\n\n");
  fprintf(fp_asc,"## Initial values for (x,y): (x(0),y(0));   Flow at time t: (x(t),y(t))\n##\n");
  /*
  fprintf(fp_asc,"## Grid of initial values\t");
  for (j=1; j<no_snapshots; j++){
    fprintf(fp_asc,"%16s%-8g\t","Flow at time t=", snapshot_times[j]);
  }
  fprintf(fp_asc,"\n");
  */
  fprintf(fp_asc,"##%6s\t%8s\t", "x(0)", "y(0)");
  for (j=1; j<no_snapshots; j++){
    fprintf(fp_asc,"x(%8g)\ty(%8g)\t", snapshot_times[j], snapshot_times[j]);
  }
  fprintf(fp_asc,"\n\n");
  for (i=0; i<no_initial_points; i++) {
    for (j=0; j<no_snapshots; j++){
      fprintf(fp_asc,"%8g\t%8g\t", xnon[i][j], ynon[i][j]);
    }
    fprintf(fp_asc,"\n");
  }
  fclose(fp_asc);
}

int main(int argc, char *argv[]) {

  unsigned long   i,k,l,no_initial_points;
  FILE            *fp_log;
  char            filename_mat[NAMESIZE], filename_asc[NAMESIZE];
  char            filename_log[NAMESIZE];
  double          time;
  
  testenv(argv[0]);
  
  initialize(argv);  
  sprintf(filename_mat, "%s.mat", user_filename);
  sprintf(filename_asc, "%s.dat", user_filename);
  sprintf(filename_log, "%s.log", user_filename);
  if ((fp_log = fopen(filename_log,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n", filename_log);fflush(stderr);
    exit(1);  
  }
  no_initial_points = (no_values_x+1)*(no_values_y+1);

  gauss_init(gauss_seed);
  time=0.0;
  for(l=1;l<no_snapshots;l++) {
    for (i=0; i<no_initial_points; i++){
      xnon[i][l] = xnon[i][l-1]; 
      ynon[i][l] = ynon[i][l-1];  
    }
    for(k=0;k<snapshot_iterations[l];k++) {
      fill_gauss(&gauss1);
      compute_j_functionals(&gauss1,dt);
      time += dt;
      for(i=0;i<no_initial_points;i++) {
	if ( (xnon[i][l] != MAX_SDE) && (ynon[i][l] != MAX_SDE) ) {
	  Brus_nonlinear_ord2_bwd_for_grid(&(xnon[i][l]),&(ynon[i][l]),dt,
					   alpha,beta,sigma);
	}
	if ( (fabs(xnon[i][l]) > MAX_SDE) || (fabs(ynon[i][l]) > MAX_SDE) ) {
	  fprintf(fp_log, "Computation of the backward flow terminated\n"); 
	  fprintf(fp_log, "for beta = %g at time = %g and the initial value (x(0),y(0))=(%8g,%8g)\n",
		  beta,-time,xnon[i][0],ynon[i][0]);
	  fprintf(fp_log, "as |x(%g)| or |y(%g)| > %8g\n",time,time,MAX_SDE);
	  xnon[i][l] = MAX_SDE*sign(xnon[i][l]);
	  ynon[i][l] = MAX_SDE*sign(ynon[i][l]);
	}
      }
    }
  }
  for(l=1;l<no_snapshots;l++) {  
    snapshot_times[l] *= -1.0;
  }
  save_data_in_matlab_format(filename_mat); 
  save_data_in_ascii_format(filename_asc);
  fclose(fp_log);

  return 0;
}




