#include <stdlib.h>
#include <stdio.h> 
#include <string.h>
#include <math.h>

#include "utils.h"
#include "j.h"
#include "brus_slv.h"


/***********  global variables  ***********/

long           gauss_seed;
double         alpha, beta, sigma;
double         dt;                    
double         x_low, x_high, y_low, y_high;
double         x_s, y_s;
double	       x_min, x_max, y_min, y_max, dx, dy;
double         **xnon, **ynon;
double         *xv, *yv;
double         *trj_x, *trj_y;
double         tmp_real;
int            no_bins_x, no_bins_y;
double         t_end;
unsigned long  iterations;
double         time_to_find_grid_dimensions;
/*  iterations to find grid dimensions  */
unsigned long  iterations_to_find_grid_dims;
double         time_for_relaxation;
unsigned long  iterations_for_relaxation;
int            grid_fixed_flag;
char           *user_filename;
int            user_filenamesize;

/***********  global variables  ***********/

void initialize (char *argv[]) {

  /*  grid_fixed_flag: 1=fixed,0=non_fixed  */

  sscanf(argv[1],  "%d",   &user_filenamesize);
  user_filename = salloc((user_filenamesize+1)*sizeof(*user_filename));
  sscanf(argv[2],  "%s",   user_filename);
  sscanf(argv[3],  "%ld",  &gauss_seed);
  sscanf(argv[4],  "%lf",  &t_end);
  sscanf(argv[5],  "%lf",  &dt);
  sscanf(argv[6],  "%lf",  &x_s); 
  sscanf(argv[7],  "%lf",  &y_s); 
  sscanf(argv[8],  "%d",   &no_bins_x); 
  sscanf(argv[9],  "%d",   &no_bins_y); 
  sscanf(argv[10], "%lf",  &x_low); 
  sscanf(argv[11], "%lf",  &x_high);
  sscanf(argv[12], "%lf",  &y_low); 
  sscanf(argv[13], "%lf",  &y_high);  
  sscanf(argv[14], "%lf",  &alpha);
  sscanf(argv[15], "%lf",  &beta);
  sscanf(argv[16], "%lf",  &sigma);
  sscanf(argv[17], "%lf",  &time_to_find_grid_dimensions);
  sscanf(argv[18], "%lf",  &time_for_relaxation);
  sscanf(argv[19], "%d",   &grid_fixed_flag); 
    
  iterations = (unsigned long) round(t_end / dt);
  iterations_to_find_grid_dims = 
    (unsigned long) round(time_to_find_grid_dimensions / dt);
  iterations_for_relaxation = 
    (unsigned long) round(time_for_relaxation / dt);
  trj_x = salloc(iterations_to_find_grid_dims*sizeof(*trj_x));
  trj_y = salloc(iterations_to_find_grid_dims*sizeof(*trj_y));
  if (DEBUG) {
    fprintf(stderr,"\nArguments for <%s>\n\n",argv[0]);
    fprintf(stderr,"user_filenamesize:             %8d\n",   user_filenamesize);
    fprintf(stderr,"user_filename:                 <%s>\n", user_filename);
    fprintf(stderr,"gauss_seed:                    %8ld\n", gauss_seed);
    fprintf(stderr,"t_end:                         %8g\n",  t_end);
    fprintf(stderr,"dt:                            %8g\n",  dt);
    fprintf(stderr,"x_s:                           %8g\n",  x_s);
    fprintf(stderr,"y_s:                           %8g\n",  y_s);
    fprintf(stderr,"no_bins_x:                     %8d\n",  no_bins_x);
    fprintf(stderr,"no_bins_y:                     %8d\n",  no_bins_y);
    fprintf(stderr,"x_low:                         %8g\n",  x_low);
    fprintf(stderr,"x_high:                        %8g\n",  x_high);
    fprintf(stderr,"y_low:                         %8g\n",  y_low);
    fprintf(stderr,"y_high:                        %8g\n",  y_high);
    fprintf(stderr,"alpha:                         %8g\n",  alpha);
    fprintf(stderr,"beta:                          %8g\n",  beta);
    fprintf(stderr,"sigma:                         %8g\n",  sigma);
    fprintf(stderr,"time_to_find_grid_dimensions:  %8g\n",  time_to_find_grid_dimensions);
    fprintf(stderr,"time_for_relaxation:           %8g\n",  time_for_relaxation);
    fprintf(stderr,"grid_fixed_flag:               %8d\n\n",  grid_fixed_flag);
    fflush(stderr);
  }
}

int make_histogram(double x, 
		   double y, 
		   double **d, 
		   int    no_x, 
		   int    no_y) {
  
  static long int  next=0;
  unsigned long    i;	
  int              x_index, y_index;
  double           width_x, width_y;
  
  if (next < iterations_to_find_grid_dims) {
    trj_x[next] = x;
    trj_y[next++] = y;
    
    if (next == iterations_to_find_grid_dims) {
      x_max = x_min = trj_x[0];
      y_max = y_min = trj_y[0];
      for(i = 1; i<iterations_to_find_grid_dims; i++) {
	if (trj_x[i] > x_max) x_max = trj_x[i];
	if (trj_x[i] < x_min) x_min = trj_x[i];
	if (trj_y[i] > y_max) y_max = trj_y[i];
	if (trj_y[i] < y_min) y_min = trj_y[i];
      }
      width_x = (x_max-x_min)/2;
      x_min -= width_x;
      x_max += width_x;
      width_y = (y_max-y_min)/2;
      y_min -= width_y;
      y_max += width_y;
      dx = (x_max-x_min)/(no_x-1);
      dy = (y_max-y_min)/(no_y-1);
      for(i=0; i<iterations_to_find_grid_dims; i++) {
	x_index = round((trj_x[i]-x_min)/dx);
	y_index = round((trj_y[i]-y_min)/dy);
	d[x_index][y_index]++;
      }
    }
    return 0;
  }
  
  if (x < x_min) x_index = 0;
  else if (x > x_max) x_index = no_x-1;
  else x_index = round((x-x_min)/dx);
  
  if (y < y_min) y_index = 0;
  else if (y > y_max) y_index = no_y-1;
  else y_index = round((y-y_min)/dy);
  
  d[x_index][y_index]++;
  
  return 1;
}

int make_histogram_fixed (double x, 
			  double y, 
			  double **d, 
			  int    no_x, 
			  int    no_y) {
  
  int x_index, y_index;
  
  dx = (x_high-x_low)/(no_x-1);
  dy = (y_high-y_low)/(no_y-1);
  
  if (x < x_low) x_index = 0;
  else if (x > x_high) x_index = no_x-1;
  else x_index = round((x-x_low)/dx);
  
  if (y < y_low) y_index = 0;
  else if (y > y_high) y_index = no_y-1;
  else y_index = round((y-y_low)/dy);
  
  d[x_index][y_index]++;
  
  return 1;
}

void save_data_in_ascii_format(double  **d, 
			       int     no_x, 
			       int     no_y, 
			       char    *fname,
			       double  nfactor) {
  FILE           *fp, *fp_lbl;
  unsigned long  i,j;

  
  fp = fopen(fname,"wt");
  if (fp == NULL) {
    fprintf(stderr,"Unable to open file %s\n",fname);fflush(stderr);
    exit(2);
  }  
  fprintf(fp, "## Parameters:\n##\n");
  fprintf(fp, "#alpha=                              %8g\n",   alpha);
  fprintf(fp, "#beta=                               %8g\n",   beta);
  fprintf(fp, "#sigma=                              %8g\n",   sigma);
  fprintf(fp, "#time_stepsize=                      %8g\n",   dt);
  fprintf(fp, "#t_end=                              %8g\n",   t_end);
  fprintf(fp, "#initial_value_x=                    %8g\n",   x_s); 
  fprintf(fp, "#initial_value_y=                    %8g\n",   y_s); 
  fprintf(fp, "#x_low=                              %8g\n",   x_low); 
  fprintf(fp, "#x_high=                             %8g\n",   x_high); 
  fprintf(fp, "#y_low=                              %8g\n",   y_low); 
  fprintf(fp, "#y_high=                             %8g\n",   y_high); 
  fprintf(fp, "#number_of_bins_x=                   %8d\n",   no_bins_x); 
  fprintf(fp, "#number_of_bins_y=                   %8d\n",   no_bins_y); 
  fprintf(fp, "#seed=                               %8ld\n",  gauss_seed);
  fprintf(fp, "#time_to_find_grid_dimensions=       %8g\n",   time_to_find_grid_dimensions);
  fprintf(fp, "#time_for_relaxation_of_trajectory=  %8g\n",   time_for_relaxation);
  fprintf(fp, "#grid_fixed=                         %8d\n",   grid_fixed_flag); 
  fprintf(fp, "#normalizing_factor=                 %8g\n",   nfactor); 
  fprintf(fp,"\n\n\n");

  fprintf(fp,"## Histogram (h[x][y]):\n##\n");
  fprintf(fp,"##%6s\t%8s\t%8s\n\n","x","y","h[x][y]");
  for(j=0; j<no_y; j++) {
    for(i=0; i<no_x; i++) {
      fprintf(fp,"%8g\t%8g\t%8g\n",x_min+i*dx, y_min+j*dy,d[i][j]);
    }
    fprintf(fp,"\n");
  }
  
  fclose(fp);
}

void save_data_in_matlab_format(double  **d, 
				int     no_x, 
				int     no_y, 
				char    *fname,
				double  nfactor) {
  FILE            *fp;
  unsigned long   i,j;
  double          *xvector, *yvector, *dvector;
  double          tmp;
  
  fp = fopen(fname,"wb");
  if (fp == NULL) {
    fprintf(stderr,"Unable to open file %s\n",fname);fflush(stderr);
    exit(2);
  }
  xvector = salloc(no_x * sizeof(*xvector));
  yvector = salloc(no_y * sizeof(*yvector));
  dvector = salloc(no_x * no_y * sizeof(*dvector));
  for(i=0; i<no_x; i++) {
    xvector[i] = x_min+i*dx;
  }
  for(i=0; i<no_y; i++) {
    yvector[i] = y_min+i*dy;
  }
  dvector = salloc(no_x * no_y * sizeof(*dvector));
  for(j=0; j<no_y; j++) {
    for(i=0; i<no_x; i++) {
      dvector[j*no_x+i] =  d[i][j];
    }
  }
  savemat(fp,TYPE_FLAG, "histogram_M",         no_x,no_y,0, dvector,                       (double *) NULL); 
  
  savemat(fp,TYPE_FLAG, "alpha",               1,1,0,       &alpha,                        (double *) NULL);
  savemat(fp,TYPE_FLAG, "beta",                1,1,0,       &beta,                         (double *) NULL);
  savemat(fp,TYPE_FLAG, "sigma",               1,1,0,       &sigma,                        (double *) NULL); 
  savemat(fp,TYPE_FLAG, "time_stepsize",       1,1,0,       &dt,                           (double *) NULL);
  savemat(fp,TYPE_FLAG, "x_coordinates_V",     1,no_x,0,    xvector,                       (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_coordinates_V",     1,no_y,0,    yvector,                       (double *) NULL); 
  tmp = log10(t_end);
  savemat(fp,TYPE_FLAG, "log10_t_end",         1,1,0,       &tmp,                          (double *) NULL); 
  savemat(fp,TYPE_FLAG, "time_find_grid_dims", 1,1,0,       &time_to_find_grid_dimensions, (double *) NULL); 
  savemat(fp,TYPE_FLAG, "time_relax_traject",  1,1,0,       &time_for_relaxation,          (double *) NULL); 
  tmp = (double ) gauss_seed;
  savemat(fp,TYPE_FLAG, "seed",                1,1,0,       &tmp,                          (double *) NULL); 
  savemat(fp,TYPE_FLAG, "initial_value_x",     1,1,0,       &x_s,                          (double *) NULL); 
  savemat(fp,TYPE_FLAG, "initial_value_y",     1,1,0,       &y_s,                          (double *) NULL); 
  tmp = (double ) no_bins_x;
  savemat(fp,TYPE_FLAG, "number_of_bins_x",    1,1,0,       &tmp,                          (double *) NULL); 
  tmp = (double ) no_bins_y;
  savemat(fp,TYPE_FLAG, "number_of_bins_y",    1,1,0,       &tmp,                          (double *) NULL); 
  savemat(fp,TYPE_FLAG, "x_low",               1,1,0,       &x_low,                        (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_low",               1,1,0,       &y_low,                        (double *) NULL); 
  savemat(fp,TYPE_FLAG, "x_high",              1,1,0,       &x_high,                       (double *) NULL); 
  savemat(fp,TYPE_FLAG, "y_high",              1,1,0,       &y_high,                       (double *) NULL); 
  tmp = (double ) grid_fixed_flag;
  savemat(fp,TYPE_FLAG, "grid_fixed",          1,1,0,       &tmp,                          (double *) NULL); 
  savemat(fp,TYPE_FLAG, "normalizing_factor",  1,1,0,       &nfactor,                      (double *) NULL); 
  fclose(fp);
}


void smooth(double         **dst, 
	    double         **src, 
	    unsigned long  no_x, 
	    unsigned long  no_y, 
	    unsigned long  width) {

  double *filter, **tmp;
  double summe;
  int i, j, s, len;
  
  len = 2*width+1;
  filter = salloc(len*sizeof(*filter));
  summe = 0;
  for(i=0; i<len; i++) {
    summe    +=
      filter[i] = width+1-abs(i-width);
  }
  for(i=0; i<len; i++)
    filter[i] /= summe;
  
  tmp    = salloc(sizeof(*tmp)*no_x);
  for(i=0; i<no_x; i++) {
    tmp[i] = salloc(sizeof(**tmp)*no_y);
  }
  
  for (i=0; i<no_x; i++) {
    for (j=0; j<no_y; j++) {
      tmp[i][j] = 0;
      for(s=i-width; s<i+width+1; s++) {
	if ((s < 0) || (s>= no_x)) 
	  continue;
	tmp[i][j] += src[s][j]*filter[s-i+width];
      }
    }
  }
  
  for (i=0; i<no_x; i++) {
    for (j=0; j<no_y; j++) {
      dst[i][j] = 0;
      for(s=j-width; s<j+width+1; s++) {
	if ((s < 0) || (s>= no_y)) 
	  continue;
	dst[i][j] += tmp[i][s]*filter[s-j+width];
      }
    }
  }
  
  for(i=0; i<no_x; i++) {
    free(tmp[i]);
  }
  free(tmp);	
}

int main(int argc, char *argv[])
{
  unsigned long  i,j,k;
  char           file_praefix[NAMESIZE];
  char 	         filename[NAMESIZE];
  FILE           *fp_log;
  double         x_old,x_new,y_old,y_new;
  double         **d, **d_smoothed;
  double         time;
  
  testenv(argv[0]);

  initialize(argv);
  strcpy(file_praefix,user_filename); 
  strcat(strcpy(filename,file_praefix),".log");
  if ((fp_log = fopen(filename,"wt")) == NULL) {
    fprintf(stderr,"Unable to open file %s\n", filename);
    exit(1);  
  }

  gauss_init(gauss_seed);
  d          = salloc(sizeof(*d)*no_bins_x);
  d_smoothed = salloc(sizeof(*d_smoothed)*no_bins_x);
  for(i=0; i<no_bins_x; i++) {
    d[i] = salloc(sizeof(**d)*no_bins_y);
    d_smoothed[i] = salloc(sizeof(**d_smoothed)*no_bins_y);
    for(j=0; j<no_bins_y; j++)
      d[i][j] = 0.0;
  } /*  d[0:no_bins_x-1][0:no_bins_y-1]*/
  x_min = x_low; 
  y_min = y_low;
  x_old = x_s;
  y_old = y_s;
  time=dt;
  for(k=0;k<iterations_for_relaxation;k++) {
    fill_gauss(&gauss1);
    compute_j_functionals(&gauss1,dt);	
    Brus_nonlinear_ord_2(x_old, &x_new, y_old, &y_new, 
			 dt, alpha, beta, sigma);
    if ( (fabs(x_new) > MAX_SDE) || (fabs(y_new) > MAX_SDE) ) {
      fprintf(fp_log, "Computation of the solution terminated\n"); 
      fprintf(fp_log, "at time = %g for beta = %g and the initial value (x(0),y(0))=(%g,%g)\n", 
	      time,beta,x_s,y_s);
      fprintf(fp_log, "as |x(%g)| or |y(%g)| > %g\n",time,time,MAX_SDE);
      exit(2);
    } 
    x_old = x_new;
    y_old = y_new;
    time += dt;
  }

  time=dt;
  for(k=0;k<iterations;k++) {
    fill_gauss(&gauss1);
    compute_j_functionals(&gauss1,dt);	
    Brus_nonlinear_ord_2(x_old, &x_new, y_old, &y_new, 
			 dt, alpha, beta, sigma); 
    if ( (fabs(x_new) > MAX_SDE) || (fabs(y_new) > MAX_SDE) ) {
      fprintf(fp_log, "Computation of the solution terminated\n"); 
      fprintf(fp_log, "at time = %g for beta = %g and the initial value (x(0),y(0))=(%g,%g)\n", 
	      time,beta,x_s,y_s);
      fprintf(fp_log, "as |x(%g)| or |y(%g)| > %g\n",time,time,MAX_SDE);
      break;
    }
    x_old = x_new;
    y_old = y_new;
    if (grid_fixed_flag) {
      make_histogram_fixed(x_new, y_new, d, no_bins_x, no_bins_y);
    } else {
      make_histogram(x_new, y_new, d, no_bins_x, no_bins_y); 
    }
    time += dt;
  }
  strcat(strcpy(filename,file_praefix),".dat");
  save_data_in_ascii_format(d,no_bins_x,no_bins_y,filename,iterations);

/*  
  smooth(d_smoothed, d, no_bins_x, no_bins_y, 2);
  strcat(strcpy(filename,file_praefix),"_sm.dat");
  save_data_in_ascii_format(d_smoothed, no_bins_x, no_bins_y,filename,iterations);
*/

  strcat(strcpy(filename,file_praefix),".mat");
  save_data_in_matlab_format(d, no_bins_x,no_bins_y, filename,iterations); 

/*
  strcat(strcpy(filename,file_praefix),"_sm.mat");
  save_data_in_matlab_format(d_smoothed, no_bins_x, no_bins_y,filename,iterations);
*/
  fclose(fp_log);

  return 0;
}








