#include <math.h>
#include <stdio.h> 
#include <string.h>

#include "oned_slv.h"
#include "utils.h"


/* Fuer Lambda_Hierarchical(4,1): ord:2  */
double           hj1, j00, j10,j110;
unsigned long    until_stationary;
random_set       gauss1;



/********************************************
Taylor-Verfahren zweiter Ordnung fuer 
dx_t = (alpha x_t + x_t^3)dt + sigma dW_t
Verwendet wird Lambda_hierarchical(4,1)
mit hj1, j00, j10,j110
********************************************/

void oned_nonlinear_ord2 (double x_start, double *x_new, 
			  double dt, double alpha, double sigma) {

  double t2,t4,t8,t12;

  t2 = x_start*x_start;
  t4 = alpha*x_start+t2*x_start;
  t8 = alpha+3.0*t2;
  t12 = sigma*sigma;
  *x_new = x_start+dt*t4+hj1*sigma+j00*t4*t8+j10*sigma*t8+6.0*j110*t12*x_start;

}


void oned_nonlinear_ord2_for_grid (double *xini,
				   double dt, 
				   double alpha, double sigma) {

  double t2,t4,t8,t12;
  double x_old = *xini;


  t2 = x_old*x_old;
  t4 = alpha*x_old+t2*x_old;
  t8 = alpha+3.0*t2;
  t12 = sigma*sigma;
  *xini = x_old+dt*t4+hj1*sigma+j00*t4*t8+j10*sigma*t8+6.0*j110*t12*x_old;

}



/* Linearized system: dV = (alpha + 3 xo^2)*V*dt ,      
   linearized in x0: Taylor: Lambda_hierarchical(4,1) */
 
void oned_linear_ord2 (double x_start, double x0, double *x_new, 
		       double dt, double alpha) {
  double t1,t2,t5;

  t1 = x0*x0;
  t2 = alpha+3.0*t1;
  t5 = t2*t2;
  *x_new = x_start+dt*t2*x_start+j00*t5*x_start;

}



/********************************************
Taylor-Verfahren zweiter Ordnung fuer 
dx_t = (-alpha x_t - x_t^3)dt + sigma dW_t
Verwendet wird Lambda_hierarchical(4,1).
********************************************/

void oned_nonlinear_ord2_fwd (double x_start,double *x_new, 
			      double dt, 
			      double alpha,double sigma) {
  double t2,t4,t8,t12;
  
  t2 = x_start*x_start;
  t4 = -alpha*x_start-t2*x_start;
  t8 = -alpha-3.0*t2;
  t12 = sigma*sigma;
  *x_new = x_start+dt*t4+hj1*sigma+j00*t4*t8+j10*sigma*t8
          -6.0*j110*t12*x_start;
}


void oned_linear_ord2_fwd (double x_start,double x0, 
			   double *x_new,double dt, 
			   double alpha) {
  double t1,t2,t5;

  t1 = x0*x0;
  t2 = -alpha-3.0*t1;
  t5 = t2*t2;
  *x_new = x_start+dt*t2*x_start+j00*t5*x_start;
}


void compute_j_functionals_oned(random_set *ptr_rs, double time_step) {

/* Fuer Lambda_Hierarchical(4,1): ord:2  begin*/
  hj1  = j_1(ptr_rs,fabs(time_step)); 
  j10  = j_10(ptr_rs,fabs(time_step)); 
  j00  = j_00(fabs(time_step)); 
  j110 = j_110(ptr_rs,fabs(time_step)); 
/*  Fuer Lambda_Hierarchical(4,1)        end*/
}

/* Compute stationary nonlinear solution */
void compute_stat_nl_solution_oned ( double beta,
				     double sigma,
				     double dt,
				     double nlx1_s,
				     double nlx2_s,
				     FILE   *fp_log,
				     int    modulus,
				     double *nlx_stat
				     ) {
  
  int       stationary_nl_solution_reached;
  int       loop;
  double    diff_x,arith_mean_x;
  double    nlx1_old=0.0,nlx2_old=0.0;
  double    nlx1_new=0.0,nlx2_new=0.0;
  double    time;
  
  nlx1_old = nlx1_s; 
  nlx2_old = nlx2_s;
  srand48(UNTIL_STATIONARY_SEED);
  stationary_nl_solution_reached = FALSE;
/*
 if (DEBUG) {
      fprintf(stderr, "Computing stationary solution for BETA= %g\n\n", beta);
      fprintf(stderr,"time \t nlx1 \t\t nlx2\n");
      fprintf(stderr,"%g  \t%.6f\t%.6f\n",0.0,nlx1_old,nlx2_old);
 }
*/
  time = dt;
  loop = 1;
  while ( (!stationary_nl_solution_reached) || 
	  (loop <= until_stationary) ) {
    fill_gauss(&gauss1);
    compute_j_functionals_oned(&gauss1,dt);	
    oned_nonlinear_ord2_fwd (nlx1_old, &nlx1_new, dt, beta, sigma); 
    oned_nonlinear_ord2_fwd (nlx2_old, &nlx2_new, dt, beta, sigma); 
    if ( fabs(nlx1_new) > MAX_SDE ) {
      fprintf(fp_log, "Computation of the stationary solution terminated\n");
      fprintf(fp_log, "at time = %g for beta = %g and the initial value x(0)=%g\n", 
	      time,beta,nlx1_s);
      fprintf(fp_log, "as |x(%g)| > %g\n",time,MAX_SDE);
      exit(2);
    }
    if ( fabs(nlx2_new) > MAX_SDE ) {
      fprintf(fp_log, "Computation of the stationary solution terminated\n");
      fprintf(fp_log, "at time = %g for beta = %g and the initial value x(0)=%g\n", 
	   time,beta,nlx2_s);
      fprintf(fp_log, "as |x(%g)| > %g\n",time,MAX_SDE);
      exit(2);
    }  
    diff_x = fabs(nlx1_new - nlx2_new);
    arith_mean_x = fabs(nlx1_new + nlx2_new)*0.5;
    if( diff_x < (EPS_DIFF_X*arith_mean_x) ){
      stationary_nl_solution_reached = TRUE;
    }
/*
    if (DEBUG) {
      if ( (loop % modulus) == 0 ) {
	fprintf(stderr,"%g  \t%.6f\t%.6f\n",time,nlx1_new,nlx2_new);
	fflush(stderr);
      }
    }
*/
    nlx1_old = nlx1_new;
    nlx2_old = nlx2_new;
    time += dt;
    loop++;
  }
  *nlx_stat = nlx1_new;
/*
  if (DEBUG) {
    fprintf(stderr,"The stationary solution for BETA=%g at time %g is:\n"
	    "%g  \t%.6f\t%.6f\n\n",
	    beta,time-dt,time-dt,nlx1_new,nlx2_new);
    fprintf(stderr, "\n\nComputing the Lyapunov exponents for BETA=%g:\n", beta);
  }
*/
}








